@@ -13,6 +13,8 @@
  * 2001/06/26 - EvB - Created
  * 2001/10/29 - EvB - Moved start_conf() from bigtest to here as
  * 		      conf_start(). Not ideal, but still the best place IMHO.
+ * 2002/03/01 - EvB - Made conf_del free interface's ACLs too
+ * 2002/03/19 - EvB - Passed module base dir to subprocesses as current dir
  */
 
 char config_id[] = "CONFIG - Copyright (C) 2001 Emile van Bergen.";
@@ -23,11 +25,12 @@ char config_id[] = "CONFIG - Copyright (C) 2001 Emile van Bergen.";
  */
 
 
-#include <malloc.h>
+#include <stdlib.h>
 #include <errno.h>
 #include <string.h>	/* For memset() */
 #include <unistd.h>	/* For close() */
-#include <arpa/inet.h>	/* For inet_ntoa */
+#include <netinet/in.h>	/* For struct in_addr */
+#include <arpa/inet.h>	/* For inet_ntoa() */
 #include <fcntl.h>
 
 #include <textfile.h>
@@ -92,7 +95,8 @@ static int add_source(CONF *c, META *m, META_AV *data)
 
 /* used by conf_new: adds interface to configuration */
 
-static int add_iface(CONF *c, META *m, META_AV *data, char *modbasepath)
+static int add_iface(CONF *c, META *m, META_AV *data, 
+		     char *modbasepath, char *progcwd)
 {
 	IFACE *iface;
 	META_SPEC spec;
@@ -198,7 +202,7 @@ static int add_iface(CONF *c, META *m, META_AV *data, char *modbasepath)
 
 		/* Create a subprocess that is associated with this channel */
 		p = proc_new(av->p, av->l, iface->flags, iface->xfertimeout, 
-			     ch, modbasepath);
+			     ch, modbasepath, progcwd);
 		if (!p) {
 			msg(F_MISC, L_ERR, "add_iface: ERROR: could not define "
 				           "subprocess '%s' for interface '%s'"
@@ -318,8 +322,11 @@ CONF *conf_new(META *m, char *basepath, char *modbasepath)
 
 		if (vm_getiface(vm) == ifs)
 			n = add_source(ret, m, vm->head[VM_REQ]);
-		else
-			n = add_iface(ret, m, vm->head[VM_REQ], modbasepath);
+		else {
+			/* We use the module base path as prog's cwd as well */
+			n = add_iface(ret, m, vm->head[VM_REQ], 
+				      modbasepath, modbasepath);
+		}
 		if (n) goto cn_cleanup;
 
 		/* As a convenience, clear the VM's lists after each call */
@@ -427,8 +434,11 @@ void conf_del(CONF *c)
 				free(ch);
 			}
 			
-			
-			/* Free interface and go to next */
+			/* Free ACLs */
+			meta_freeavlist(c->ifaces->sendacl);
+			meta_freeavlist(c->ifaces->recvacl);
+
+			/* Free interface itself and go to next */
 			i = c->ifaces->next; free(c->ifaces); c->ifaces = i;
 		}
 
