@@ -25,7 +25,7 @@ char jobs_id[] = "JOBS - Copyright (C) 2001 Emile van Bergen.";
 
 #include <string.h>
 #include <errno.h>
-#include <malloc.h>
+#include <stdlib.h>
 
 #include <language.h>		/* Also includes metaops.h */
 #include <config.h>		/* For conf_new() etc */
@@ -67,7 +67,7 @@ JOB *job_new(SOCK *s, time_t t)
 	/* Allocate new job, including packet buffer */
 	j = (JOB *)malloc(sizeof(JOB)); if (!j) return 0;
 	memset(j, 0, sizeof(JOB));
-	j->pkt = (char *)malloc(C_MAX_PKTSIZE + C_MAX_SECRSIZE); 
+	j->pkt = (char *)malloc(C_MAX_PKTSIZE);
 	if (!j->pkt) { free(j); return 0; }
 
 	/* Receive packet, setting also the reply fd and sockaddr_in */
@@ -79,6 +79,11 @@ JOB *job_new(SOCK *s, time_t t)
 				   strerror(errno)); job_del(j); return 0; }
 	j->replyfd = s->fd;
 
+	if (msg_thresh[F_RECV] >= L_NOTICE) {
+		msg(F_RECV, L_NOTICE, "job_new: Received request:\n"); 
+		hexdump(j->pkt, j->pktlen);
+	}
+
 	/* Decode it. Note that the pointers in the AV items may refer to
 	   the data in the packet buffer, so don't free that packet buffer. */
 	reqh = meta_decode(s->c->m, s->c->ds_rad_pkt, 0, j->pkt, j->pktlen, 
@@ -105,11 +110,8 @@ JOB *job_new(SOCK *s, time_t t)
 	j->vm = vm_new(s->c->expr, s->c->exprlen, C_MAX_STKDEPTH,reqh,reqt,0,0);
 
 	/* Show things */
-	if (msg_thresh[F_RECV] >= L_NOTICE) {
-		msg(F_RECV, L_NOTICE, "job_new: Received request:\n"); 
-		hexdump(j->pkt, j->pktlen);
-		if (msg_thresh[F_RECV] >= L_DEBUG)
-			meta_printavlist(s->c->m, j->vm->head[VM_REQ], 0);
+	if (msg_thresh[F_RECV] >= L_DEBUG) {
+		meta_printavlist(s->c->m, j->vm->head[VM_REQ], 0);
 	}
 
 	return j;
@@ -140,6 +142,7 @@ void job_del(JOB *j)
 IFACE *job_run(CONF *c, JOB *j)
 {
 	META_AV *av, *avtree;
+	md5_state_t mds;
 	IFACE *ret;
 	int n;
 
@@ -175,9 +178,10 @@ IFACE *job_run(CONF *c, JOB *j)
 				       "Could not encode response!\n"); break; }
 
 		/* Sign the packet using the secret described by 'av' */
-		memcpy(j->pkt + j->pktlen, av->p, av->l);
-		md5(j->pkt + c->di_authenticator->val_ofs,
-		    j->pkt, j->pktlen + av->l);
+		md5_init(&mds);
+		md5_append(&mds, (unsigned char *)j->pkt, j->pktlen);
+		md5_append(&mds, (unsigned char *)av->p, av->l);
+		md5_finish(&mds, (unsigned char *)j->pkt + c->di_authenticator->val_ofs);
 
 		/* Send the reply */
 		if (msg_thresh[F_SEND] >= L_NOTICE) hexdump(j->pkt, j->pktlen);
@@ -221,7 +225,7 @@ void job_toiface(IFACE *i, JOB *j, time_t t)
 	   send queue. */
 	for(ch = i->rrch; ; ) {
 
-		/* If we found an idle channel, make it take the job */
+		/* If we found a channel with idle tx, make it take the job */
 		if (ch->proc->state == PRS_IDLE || 
 		    ch->proc->state == PRS_RECEIVING) {
 			job_tochan(ch, j, t);
@@ -236,8 +240,13 @@ void job_toiface(IFACE *i, JOB *j, time_t t)
 		if (ch == i->rrch) break;
 	}
 
-	msg(F_PROC, L_NOTICE, "job_toiface: Putting job on shared queue for "
-			      "interface '%s'.\n", i->name);
+	if (i->sendqlen >= C_MAX_SENDQLEN) {
+		msg(F_PROC, L_NOTICE, "job_toiface: Shared queue for interface "
+				      "%s overflowing (%d) - dropping job!\n",
+				      i->sendqlen);
+		job_del(j);
+		return;
+	}
 
 	/* Put job on this interface's shared send queue */
 	j->next = 0;				/* my next = none */
@@ -245,6 +254,10 @@ void job_toiface(IFACE *i, JOB *j, time_t t)
 	if (i->sendqh) i->sendqh->next = j;	/* current head's next = me */
 	else i->sendqt = j;			/* if none, then I'm tail too */
 	i->sendqh = j;				/* I am always the new head */
+	i->sendqlen++;
+
+	msg(F_PROC, L_NOTICE, "job_toiface: Job put on shared queue for "
+			      "interface '%s' (%d)\n", i->name, i->sendqlen);
 }
 
 
@@ -254,12 +267,10 @@ void job_tochan(CHAN *ch, JOB *j, time_t t)
 	ssize_t msglen;
 	PROC *p;
 
-	/* Get the channel's process and put it in motion if it was idle */
+	/* Put the channel's process tx'er in motion; apparently it was idle */
 	p = ch->proc;
 	p->state |= PRS_SENDING;
 	p->timer = t + p->xfertimeout;
-	msg(F_PROC, L_DEBUG, "- set sending state on %s's %s, pid %d\n",
-	    ch->iface->name, p->argv[0], p->pid);
 
 	/* Create the message from the request list. */
 	msglen = meta_avtomsg(ch->iface->c->m, j->vm->head[VM_REQ],
@@ -272,6 +283,10 @@ void job_tochan(CHAN *ch, JOB *j, time_t t)
 	   printav(), which would also benefit from that... */
 	ring_put(p->w, reqmsg, msglen);
 
+	msg(F_PROC, L_DEBUG, "- set sending state on %s's pid %d, %d now in "
+			     "ring\n", 
+	    ch->iface->name, p->pid, ring_maxget(p->w));
+
 	/* Put job at head of channel's recv queue */
 	j->next = 0;				/* my next = none */
 	j->prev = ch->recvqh;			/* my prev = current head */
